#!/usr/bin/perl -w
#
# 1.basic.t
#
# LDAP ACL tests
#
# $Id$

# The vars file that defines where the LDAP server is etc:
#
my $varsFile = "../vars";

# This opens a number of LDAP connections and binds them as different
# users. It uses the DSA Manager connection to manipulate entries
# and the others to test access to them.
#
# The connections are:
#
#	$ldapMgr		DSA Manager (all powerful)
#	$ldapClerkA		The admin clerk in dept A
#	$ldapUserA		A normal user in dept A
#	$ldapAnon		An anonymous user
#

use strict;

use Carp;
use Test::Simple tests => 30;
use MyConfig;
use Data::Dumper;

# Import all constants from Net::LDAP as well as procedures
use Net::LDAP qw(:all);
use Net::LDAP::Entry;

########################################################################
# Start of tests
########################################################################

# To receive LDAP result messages
my $msg;

########################################################################
# Read config file
#
my $conf = MyConfig::readConfig($varsFile);
ok (defined($conf->{servername}), "Read the vars file");

my $ldapURI = sprintf "ldap://%s:%s/", $conf->{servername}, $conf->{serverport};

########################################################################
# Construct DNs
#
my $ApeopleDN = "dc=people,dc=a,dc=example,dc=org";
my $BpeopleDN = "dc=people,dc=b,dc=example,dc=org";
#
# And for groups
#
my $AgroupsDN = "dc=groups,dc=a,dc=example,dc=org";
my $BgroupsDN = "dc=groups,dc=b,dc=example,dc=org";
#
# Principals
#
my $ClerkA = "uid=a1,$ApeopleDN";
my $UserA = "uid=a2,$ApeopleDN";
my $ClerkB = "uid=b1,$BpeopleDN";
my $UserB = "uid=b2,$BpeopleDN";


########################################################################
# Setting up LDAP connections
########################################################################

########################################################################
# LDAP connect
#
my $ldapMgr = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapMgr, "Connecting to LDAP server $ldapURI");

########################################################################
# Read base object as ANON - it must contain our base OC
#
$msg = $ldapMgr->search(
        base => $conf->{basedn},
        filter => "(objectClass=organization)",
        scope => "base",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Reading the base object from the DIT");
my $baseObject = $msg->entry(0);

########################################################################
# Bind as DSA Manager
#
$msg = $ldapMgr->bind( $conf->{manager}, password => $conf->{password});
ok ( !$msg->is_error, "Bind as DSA Manager" );

########################################################################
# Open another LDAP connection and bind anonymously
#
my $ldapAnon = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapAnon, "Connecting to LDAP server $ldapURI");
$msg = $ldapAnon->bind;
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Bind anonymously" );

########################################################################
# Open another LDAP connection and bind as Clerk A
#
my $ldapClerkA = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapClerkA, "Connecting to LDAP server $ldapURI");
$msg = $ldapClerkA->bind( $ClerkA, password => "secret" );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Bind as Clerk A" );

########################################################################
# Open another LDAP connection and bind as User A
#
my $ldapUserA = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapUserA, "Connecting to LDAP server $ldapURI");
$msg = $ldapUserA->bind( $UserA, password => "secret" );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Bind as User A" );

########################################################################
# Clerk A reading an entry
#
$msg = $ldapClerkA->search(
        base => $conf->{basedn},
        filter => "(uid=a2)",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Clerk A should be able to find user a2 by uid");
my $foundObject = $msg->entry(0);

ok (($foundObject->get_value("uid") eq "a2"), "Clerk A should be able to read a2's uid");
ok ((not $foundObject->get_value("userPassword")), "Clerk A should not be able to read a2's password");

########################################################################
# Clerk A modifying an account entry
#
$foundObject->replace( "description", "My A2 account" );
$msg = $foundObject->update( $ldapClerkA );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Clerk A should be able to modify an account entry in Dept A" );

########################################################################
# Tests for entry creation
########################################################################

# Create a new entry

my $newEntry;

sub newPerson {
	my ($parentDN, $newName) =@_;

	$newName = "A.Person" if not $newName;

	my $Entry = Net::LDAP::Entry->new;
	$Entry->changetype( 'add' );

	$Entry->dn( "cn=$newName,$parentDN" );

	$Entry->add( "objectClass", "inetOrgPerson" );
	$Entry->add( "cn", $newName );
	$Entry->add( "sn", "Person" );
	$Entry->add( "userPassword", "secret" );

	return $Entry;
}

sub newGroup {
	my ($parentDN) =@_;

	my $Entry = Net::LDAP::Entry->new;
	$Entry->changetype( 'add' );

	$Entry->dn( "cn=Cyan,$parentDN" );

	$Entry->add( "objectClass", [ 'groupOfNames' ] );
	$Entry->add( "cn", "Cyan" );
	$Entry->add( "member", "cn=null" );

	return $Entry;
}


########################################################################
# Clerk A creating a person with extra attributes under People
#
$newEntry = newPerson($ApeopleDN, "Bad One");
my $newPersonDN = $newEntry->dn;

# Remove it in case it was already there
$msg = $ldapMgr->delete( $newEntry->dn );
carp "ldapMgr: " . $msg->error if ($msg->is_error and ($msg->error_name ne "LDAP_NO_SUCH_OBJECT"));

$newEntry->add( "objectClass", "shadowAccount" );
$newEntry->add( "shadowMin", "1" );
$newEntry->add( "uid", "attacker" );

# IBM ITDS-specific attack
# $newEntry->add( "entryOwner", "access-id:$ClerkA" );
# $newEntry->add( "ibm-filterAclEntry", "group:cn=clerks,dc=groups,dc=b,dc=example,dc=org:(objectclass=*):critical:rwsc" );
# print "Created $newPersonDN\n";

$msg = $newEntry->update( $ldapClerkA );
carp $msg->error_name if ($msg->is_error and ($msg->error_name ne "LDAP_INSUFFICIENT_ACCESS"));
ok ( ($msg->is_error and ($msg->error_name eq "LDAP_INSUFFICIENT_ACCESS")),
	"Clerk A should not be able to create person with extra attributes" );

# Remove it again
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;


########################################################################
# Clerk A creating a person under People
#
$newEntry = newPerson($ApeopleDN, "Good Boy");
$newPersonDN = $newEntry->dn;

# Remove it just in case it was left from previous tests
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

$msg = $newEntry->update( $ldapClerkA );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Clerk A creating new person entry under $ApeopleDN" );

########################################################################
# Clerk A deleting a person under People
#
$newEntry = newPerson($ApeopleDN, "Victim");

# Create the victim
$msg = $newEntry->update( $ldapMgr );
carp "Manager cannot create victim: " . $msg->error_name
	if ($msg->is_error and ($msg->error_name ne "LDAP_ALREADY_EXISTS"));

# Remove it
$msg = $ldapClerkA->delete( $newEntry->dn );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Clerk A deleting a person entry under $ApeopleDN" );


########################################################################
# Clerk A creating a group under People
#
$newEntry = newGroup($ApeopleDN);

# Remove it just in case it was left from previous tests
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

$msg = $newEntry->update( $ldapClerkA );
# carp $msg->error_name if $msg->is_error;
ok ( ($msg->is_error and ($msg->error_name eq "LDAP_INSUFFICIENT_ACCESS")),
	"Clerk A should not be able to create a new group entry under $ApeopleDN" );

# Remove it again
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;



########################################################################
# Clerk A creating a group under Groups
#
$newEntry = newGroup($AgroupsDN);

# Remove it just in case it was left from previous tests
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

$msg = $newEntry->update( $ldapClerkA );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Clerk A creating new group entry under $AgroupsDN" );

########################################################################
# Clerk A creating a person under Groups
#
$newEntry = newPerson($AgroupsDN, "Lost Person");

# Remove it just in case it was left from previous tests
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

$msg = $newEntry->update( $ldapClerkA );
# carp $msg->error_name if $msg->is_error;
ok ( ($msg->is_error and ($msg->error_name eq "LDAP_INSUFFICIENT_ACCESS")),
	"Clerk A should not be able to create a new person entry under $AgroupsDN" );

# Remove it again
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

########################################################################
# Clerk A creating a person under a different department
#
$newEntry = newPerson($BpeopleDN, "Interloper");

# Remove it just in case it was left from previous tests
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

$msg = $newEntry->update( $ldapClerkA );
# carp $msg->error_name if $msg->is_error;
ok ( ($msg->is_error and ($msg->error_name eq "LDAP_INSUFFICIENT_ACCESS")),
	"Clerk A should not be able to create a new person in dept B under $BpeopleDN" );


########################################################################
# Anon reading a person
#
$msg = $ldapAnon->search(
        base => $conf->{basedn},
        filter => "(&(objectClass=account)(uid=b1))",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Anon searching for an account by uid");
$foundObject = $msg->entry(0);

ok (($foundObject->get_value("uid") eq "b1"), "Anon should be able to read an account's uid");
ok ((not $foundObject->get_value("userPassword")), "Anon should not be able to read a person's password");

########################################################################
# Open another LDAP connection and bind as the person we created
#
my $ldapPerson = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapPerson, "Connecting to LDAP server $ldapURI");
$msg = $ldapPerson->bind( $newPersonDN, password => "secret");
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Bind as new person $newPersonDN" );

########################################################################
# Person reading own entry
#
$msg = $ldapPerson->search(
        base => $conf->{basedn},
        filter => "(&(objectClass=person)(cn=Good Boy))",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Person searching for self by name");
$foundObject = $msg->entry(0);

ok (($foundObject->get_value("sn") eq "Person"), "Person should be able to read own surname");
ok ((not $foundObject->get_value("userPassword")), "Person should not be able to read own password");

########################################################################
# Person setting own password
#
$foundObject->replace( "userPassword" => "mine!" );
$msg = $foundObject->update( $ldapPerson );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Person setting own password" );

