#!/usr/bin/perl -w
#
# 1.basic.t
#
# LDAP ACL tests
#
# $Id$

# The vars file that defines where the LDAP server is etc:
#
my $varsFile = "../vars";

# This opens a number of LDAP connections and binds them as different
# users. It uses the DSA Manager connection to manipulate entries
# and the others to test access to them.
#
# The connections are:
#
#	$ldapMgr		DSA Manager (all powerful)
#	$ldapClerkA		The admin clerk in dept A
#	$ldapUserA1		A normal user in dept A
#	$ldapAnon		An anonymous user
#

use strict;

use Carp;
use Test::Simple tests => 22;
use MyConfig;
use Data::Dumper;

# Import all constants from Net::LDAP as well as procedures
use Net::LDAP qw(:all);
use Net::LDAP::Entry;

########################################################################
# Start of tests
########################################################################

# To receive LDAP result messages
my $msg;

########################################################################
# Read config file
#
my $conf = MyConfig::readConfig($varsFile);
ok (defined($conf->{servername}), "Read the vars file");

my $ldapURI = sprintf "ldap://%s:%s/", $conf->{servername}, $conf->{serverport};

########################################################################
# Construct DNs
#
my $ApeopleDN = "dc=people,dc=a,dc=example,dc=org";
my $BpeopleDN = "dc=people,dc=b,dc=example,dc=org";
#
# And for groups
#
my $AgroupsDN = "dc=groups,dc=a,dc=example,dc=org";
my $BgroupsDN = "dc=groups,dc=b,dc=example,dc=org";
#
# Principals
#
my $UserA1 = "uid=a1,$ApeopleDN";
my $UserB = "uid=b1,$BpeopleDN";


########################################################################
# Setting up LDAP connections
########################################################################

########################################################################
# LDAP connect
#
my $ldapMgr = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapMgr, "Connecting to LDAP server $ldapURI");

########################################################################
# Read base object as ANON - it must contain our base OC
#
$msg = $ldapMgr->search(
        base => $conf->{basedn},
        filter => "(objectClass=organization)",
        scope => "base",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Reading the base object from the DIT");
my $baseObject = $msg->entry(0);

########################################################################
# Bind as DSA Manager
#
$msg = $ldapMgr->bind( $conf->{manager}, password => $conf->{password});
ok ( !$msg->is_error, "Bind as DSA Manager" );

########################################################################
# Open another LDAP connection and bind anonymously
#
my $ldapAnon = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapAnon, "Connecting to LDAP server $ldapURI");
$msg = $ldapAnon->bind;
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Bind anonymously" );

########################################################################
# Open another LDAP connection and bind as User a1
#
my $ldapUserA1 = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapUserA1, "Connecting to LDAP server $ldapURI");
$msg = $ldapUserA1->bind( $UserA1, password => "secret" );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Bind as User a1" );

########################################################################
# User a1 reading an entry
#
$msg = $ldapUserA1->search(
        base => $conf->{basedn},
        filter => "(uid=a2)",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User a1 should be able to find user a2 by uid");
my $foundObject = $msg->entry(0);

ok (($foundObject and ($foundObject->get_value("uid") eq "a2")), "User a1 should be able to read a2's uid");
ok (($foundObject and (not $foundObject->get_value("userPassword"))), "User a1 should not be able to read a2's password");


########################################################################
# A1 searching for own dept
#
$msg = $ldapUserA1->search(
        base => $conf->{basedn},
        filter => "(&(objectClass=organizationalUnit)(dc=a))",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User a1 should be able to find department A by dc");

########################################################################
# A1 searching for another dept
#
$msg = $ldapUserA1->search(
        base => $conf->{basedn},
        filter => "(&(objectClass=organizationalUnit)(dc=b))",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User a1 should be able to find department B by dc");

########################################################################
# Anon searching for dept A
#
$msg = $ldapAnon->search(
        base => $conf->{basedn},
        filter => "(&(objectClass=organizationalUnit)(dc=a))",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Anon should be able to find department A by dc");

########################################################################
# Anon searching for dept B
#
$msg = $ldapAnon->search(
        base => $conf->{basedn},
        filter => "(&(objectClass=organizationalUnit)(dc=b))",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Anon should be able to find department B by dc");

########################################################################
# Anon searching for something under a dept
#
$msg = $ldapAnon->search(
        base => $conf->{basedn},
        filter => "(&(objectClass=organizationalUnit)(dc=people))",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 0, "Anon should not be able to find a sub-department by dc");

########################################################################
# Anon reading a person
#
$msg = $ldapAnon->search(
        base => $conf->{basedn},
        filter => "(&(objectClass=account)(uid=b1))",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 0, "Anon should not be able to search for an account by uid");

########################################################################
# User a1 reading a person in dept B
#
$msg = $ldapUserA1->search(
        base => $conf->{basedn},
        filter => "(uid=b1)",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 0, "User a1 should not be able to search for an account in department B by uid");


########################################################################
# User reading own entry
#
$msg = $ldapUserA1->search(
        base => $UserA1,
        filter => "(objectClass=*)",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User reading own entry");
$foundObject = $msg->entry(0);

ok (($foundObject->get_value("uid") eq "a1"), "User a1 should be able to read own uid");
ok ((not $foundObject->get_value("userPassword")), "User a1 should not be able to read own password");

########################################################################
# UserA1 setting own password
#
$foundObject->replace( "userPassword" => "mine!" );
$msg = $foundObject->update( $ldapUserA1 );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "User a1 setting own password" );

# Set it back again
$foundObject->replace( "userPassword" => "secret" );
$msg = $foundObject->update( $ldapMgr );
carp "ldapAdm: " . $msg->error if $msg->is_error;

