#!/usr/bin/perl -w
#
# 1.basic.t
#
# LDAP ACL tests
#
# $Id$

# The vars file that defines where the LDAP server is etc:
#
my $varsFile = "../vars";

# This opens a number of LDAP connections and binds them as different
# users. It uses the DSA Manager connection to manipulate entries
# and the others to test access to them.
#
# The connections are:
#
#	$ldapMgr		DSA Manager (all powerful)
#	$ldapU1			A User
#	$ldapAnon		An anonymous user
#

use strict;

use Carp;
use Test::Simple tests => 20;
use MyConfig;
use Data::Dumper;

# Import all constants from Net::LDAP as well as procedures
use Net::LDAP qw(:all);
use Net::LDAP::Entry;

########################################################################
# Start of tests
########################################################################

# To receive LDAP result messages
my $msg;

########################################################################
# Read config file
#
my $conf = MyConfig::readConfig($varsFile);
ok (defined($conf->{servername}), "Read the vars file");

my $ldapURI = sprintf "ldap://%s:%s/", $conf->{servername}, $conf->{serverport};

my $peopleDN = "dc=people," . $conf->{basedn};

########################################################################
# Setting up LDAP connections
########################################################################

########################################################################
# LDAP connect
#
my $ldapMgr = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapMgr, "Connecting to LDAP server $ldapURI");

########################################################################
# Read base object - it must contain our base OC
#
$msg = $ldapMgr->search(
        base => $conf->{basedn},
        filter => "(objectClass=organization)",
        scope => "base",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Reading the base object from the DIT");
my $baseObject = $msg->entry(0);

########################################################################
# Bind as DSA Manager
#
$msg = $ldapMgr->bind( $conf->{manager}, password => $conf->{password});
ok ( !$msg->is_error, "Bind as DSA Manager" );

########################################################################
# Open another LDAP connection and bind anonymously
#
my $ldapAnon = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapAnon, "Connecting to LDAP server $ldapURI");
$msg = $ldapAnon->bind;
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Bind anonymously" );

########################################################################
# Open another LDAP connection and bind as a user
#
my $ldapU1 = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapU1, "Connecting to LDAP server $ldapURI");
$msg = $ldapU1->bind( "uid=u1,dc=people,dc=example,dc=org",
	password => "secret" );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Bind as User 1" );

########################################################################
# Tests for entry creation
########################################################################

# Create a new entry

my $newEntry;

sub newPerson {
	my ($parentDN) =@_;

	my $Entry = Net::LDAP::Entry->new;
	$Entry->changetype( 'add' );

	$Entry->dn( "cn=A.Person,$parentDN" );

	$Entry->add( "objectClass", "inetOrgPerson" );
	$Entry->add( "cn", "A.Person" );
	$Entry->add( "sn", "Person" );
	$Entry->add( "userPassword", "secret" );

	return $Entry;
}

########################################################################
# User creating a person entry
#
$newEntry = newPerson($peopleDN);
my $newPersonDN = $newEntry->dn;

# Remove it just in case it was left from previous tests
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

$msg = $newEntry->update( $ldapU1 );
# carp $msg->error_name if $msg->is_error;
ok ( ($msg->is_error and ($msg->error_name eq "LDAP_INSUFFICIENT_ACCESS")),
	"User should not be able to create person entry" );

# Remove it again
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

########################################################################
# Anon reading a person
#
$msg = $ldapAnon->search(
        base => $conf->{basedn},
        filter => "(uid=u1)",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Anon searching for a person by uid");
my $foundObject = $msg->entry(0);

ok (($foundObject->get_value("description") eq "A User"), "Anon should be able to read an account's description");
ok ((not $foundObject->get_value("userPassword")), "Anon should not be able to read a password");

########################################################################
# Person reading own entry
#
$msg = $ldapU1->search(
        base => $conf->{basedn},
        filter => "(uid=u1)",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User searching for self by uid");
$foundObject = $msg->entry(0);

ok (($foundObject->get_value("description") eq "A User"), "User should be able to read own description");
ok ((not $foundObject->get_value("userPassword")), "User should not be able to read own password");

########################################################################
# User setting own password
#
$foundObject->replace( "userPassword" => "ALL!mine!" );
$msg = $foundObject->update( $ldapU1 );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "User setting own password" );

# Set it back for next test
$foundObject->replace( "userPassword" => "secret" );
$msg = $foundObject->update( $ldapMgr );
carp "ldapMgr: " . $msg->error if $msg->is_error;


########################################################################
# User reading another user's entry
#
$msg = $ldapU1->search(
        base => $conf->{basedn},
        filter => "(uid=u2)",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User searching for another user by uid");
$foundObject = $msg->entry(0);

ok (($foundObject->get_value("description") eq "Another User"), "User should be able to read another user's description");
ok ((not $foundObject->get_value("userPassword")), "User should not be able to read another user's password");

########################################################################
# User setting another user's password
#
$foundObject->replace( "userPassword" => "mine!" );
$msg = $foundObject->update( $ldapU1 );
# carp $msg->error_name if $msg->is_error;
ok ( ($msg->is_error and ($msg->error_name eq "LDAP_INSUFFICIENT_ACCESS")), "User should not be able to set another user's password" );

# Set it back for next test
$foundObject->replace( "userPassword" => "secret" );
$msg = $foundObject->update( $ldapMgr );
carp "ldapMgr: " . $msg->error if $msg->is_error;

