#!/usr/bin/perl -w
#
# 1.basic.t
#
# LDAP ACL tests
#
# $Id$

# The vars file that defines where the LDAP server is etc:
#
my $varsFile = "../vars";

# This opens a number of LDAP connections and binds them as different
# users. It uses the DSA Manager connection to manipulate entries
# and the others to test access to them.
#
# The connections are:
#
#	$ldapMgr		DSA Manager (all powerful)
#	$ldapAdmin		A data admin
#	$ldapAnon		An anonymous user
#

use strict;

use Carp;
use Test::Simple tests => 22;
use MyConfig;
use Data::Dumper;

# Import all constants from Net::LDAP as well as procedures
use Net::LDAP qw(:all);
use Net::LDAP::Entry;

########################################################################
# Start of tests
########################################################################

# To receive LDAP result messages
my $msg;

########################################################################
# Read config file
#
my $conf = MyConfig::readConfig($varsFile);
ok (defined($conf->{servername}), "Read the vars file");

my $ldapURI = sprintf "ldap://%s:%s/", $conf->{servername}, $conf->{serverport};


########################################################################
# Setting up LDAP connections
########################################################################

########################################################################
# LDAP connect
#
my $ldapMgr = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapMgr, "Connecting to LDAP server $ldapURI");

########################################################################
# Read base object - it must contain our base OC
#
$msg = $ldapMgr->search(
        base => $conf->{basedn},
        filter => "(objectClass=organization)",
        scope => "base",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Reading the base object from the DIT");
my $baseObject = $msg->entry(0);

########################################################################
# Bind as DSA Manager
#
$msg = $ldapMgr->bind( $conf->{manager}, password => $conf->{password});
ok ( !$msg->is_error, "Bind as DSA Manager" );

########################################################################
# Construct DN for 'people' arc
#
my $peopleDN = "dc=people,dc=example,dc=org";
#
# And for groups
#
my $groupsDN = "dc=groups,dc=example,dc=org";

########################################################################
# Open another LDAP connection and bind anonymously
#
my $ldapAnon = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapAnon, "Connecting to LDAP server $ldapURI");
$msg = $ldapAnon->bind;
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Bind anonymously" );

########################################################################
# Open another LDAP connection and bind as data admin
#
my $ldapAdmin = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapAdmin, "Connecting to LDAP server $ldapURI");
$msg = $ldapAdmin->bind( "uid=admin,dc=people,dc=example,dc=org",
	password => "secret" );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Bind as data admin" );

########################################################################
# Tests for entry creation
########################################################################

# Create a new entry

my $newEntry;

sub newPerson {
	my ($parentDN) =@_;

	my $Entry = Net::LDAP::Entry->new;
	$Entry->changetype( 'add' );

	$Entry->dn( "cn=A.Person,$parentDN" );

	$Entry->add( "objectClass", "inetOrgPerson" );
	$Entry->add( "cn", "A.Person" );
	$Entry->add( "sn", "Person" );
	$Entry->add( "userPassword", "secret" );

	return $Entry;
}

sub newGroup {
	my ($parentDN) =@_;

	my $Entry = Net::LDAP::Entry->new;
	$Entry->changetype( 'add' );

	$Entry->dn( "cn=Cyan,$parentDN" );

	$Entry->add( "objectClass", [ 'groupOfNames' ] );
	$Entry->add( "cn", "Cyan" );
	$Entry->add( "member", "cn=null" );

	return $Entry;
}


########################################################################
# admin creating a person with extra attributes under People
#
$newEntry = newPerson($peopleDN);

$newEntry->add( "objectClass", "shadowAccount" );
$newEntry->add( "shadowMin", "1" );
$newEntry->add( "uid", "attacker" );

# Remove it just in case it was left from previous tests
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

$msg = $newEntry->update( $ldapAdmin );
# carp $msg->error_name if $msg->is_error;
ok ( ($msg->is_error and (($msg->error_name eq "LDAP_INSUFFICIENT_ACCESS") or ($msg->error_name eq "LDAP_OBJECT_CLASS_VIOLATION"))),
	"Admin should not be able to create person with extra attributes" );

# Remove it again
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;


########################################################################
# admin creating a person with extra objectclass under People
#
$newEntry = newPerson($peopleDN);

$newEntry->add( "objectClass", "userSecurityInformation" );

# Remove it just in case it was left from previous tests
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

$msg = $newEntry->update( $ldapAdmin );
# carp $msg->error_name if $msg->is_error;
ok ( ($msg->is_error and (($msg->error_name eq "LDAP_INSUFFICIENT_ACCESS") or ($msg->error_name eq "LDAP_OBJECT_CLASS_VIOLATION"))),
	"Admin should not be able to create person with extra auxiliary objectclass" );

# Remove it again
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;


########################################################################
# admin creating a person under People
#
$newEntry = newPerson($peopleDN);

# Remove it just in case it was left from previous tests
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

$msg = $newEntry->update( $ldapAdmin );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Admin creating new person entry under dc=people" );


########################################################################
# admin creating a group under People
#
$newEntry = newGroup($peopleDN);

# Remove it just in case it was left from previous tests
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

$msg = $newEntry->update( $ldapAdmin );
# carp $msg->error_name if $msg->is_error;
ok ( ($msg->is_error and ($msg->error_name eq "LDAP_INSUFFICIENT_ACCESS")),
	"Admin should not be able to create a new group entry under dc=people" );

# Remove it again
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;



########################################################################
# admin creating a group under Groups
#
$newEntry = newGroup($groupsDN);

# Remove it just in case it was left from previous tests
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

$msg = $newEntry->update( $ldapAdmin );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Admin creating new group entry under dc=groups" );


########################################################################
# admin creating a person under Groups
#
$newEntry = newPerson($groupsDN);

# Remove it just in case it was left from previous tests
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

$msg = $newEntry->update( $ldapAdmin );
# carp $msg->error_name if $msg->is_error;
ok ( ($msg->is_error and ($msg->error_name eq "LDAP_INSUFFICIENT_ACCESS")),
	"Admin should not be able to create a new person entry under dc=groups" );

# Remove it again
$msg = $ldapMgr->delete( $newEntry->dn );
# carp $msg->error if $msg->is_error;

########################################################################
# Anon reading a person
#
$msg = $ldapAnon->search(
        base => $conf->{basedn},
        filter => "(&(objectClass=person)(cn=A.Person))",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Anon searching for a person by name");
my $foundObject = $msg->entry(0);

ok (($foundObject->get_value("sn") eq "Person"), "Anon should be able to read a person's surname");
ok ((not $foundObject->get_value("userPassword")), "Anon should not be able to read a person's password");

########################################################################
# Open another LDAP connection and bind as the person we created
#
my $ldapPerson = Net::LDAP->new( $ldapURI,
                        version => 3,
			onerror => undef );

ok ($ldapPerson, "Connecting to LDAP server $ldapURI");
$msg = $ldapPerson->bind( "cn=A.Person,dc=people,dc=example,dc=org", password => "secret");
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Bind as new person" );

########################################################################
# Person reading own entry
#
$msg = $ldapPerson->search(
        base => $conf->{basedn},
        filter => "(&(objectClass=person)(cn=A.Person))",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Person searching for self by name");
$foundObject = $msg->entry(0);

ok (($foundObject->get_value("sn") eq "Person"), "Person should be able to read own surname");
ok ((not $foundObject->get_value("userPassword")), "Person should not be able to read own password");

########################################################################
# Person setting own password
#
$foundObject->replace( "userPassword" => "mine!" );
$msg = $foundObject->update( $ldapPerson );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Person setting own password" );

