#!/usr/bin/perl -w
#
# 1.basic.t
#
# LDAP ACL tests for subset-visibility
#
# $Id$

# The vars file that defines where the LDAP server is etc:
#
my $varsFile = "../vars";

# This opens a number of LDAP connections and binds them as different
# users. It uses the DSA Manager connection to manipulate entries
# and the others to test access to them.
#
# The connections are:
#
#	$ldapMgr		DSA Manager (all powerful)
#	$ldapUserA1		A normal user in dept A
#	$ldapUserA2		A normal user in dept A
#	$ldapUserB1		A normal user in dept B
#	$ldapUserB2		A normal user in dept B
#	$ldapAnon		An anonymous user
#

use strict;

use Carp;
use Test::Simple tests => 68;
use MyConfig;
use Data::Dumper;

# Import all constants from Net::LDAP as well as procedures
use Net::LDAP qw(:all);
use Net::LDAP::Entry;

# Global config hashref
my $conf;

########################################################################
# Procedures
########################################################################
#
# Make a new LDAP connection and bind it as a user
#
# my $ldapHandle=newConn( name, dn, password );
#
sub newConn {
	my ($name, $dn, $pw) = @_;

	my $handle = Net::LDAP->new( $conf->{serveruri},
				version => 3,
				onerror => undef );

	ok ($handle, "Connecting to LDAP server " . $conf->{serveruri});
	my $msg = $handle->bind( $dn, password => $pw);
	ok ( !$msg->is_error, "Bind as $name" );

	return undef if $msg->is_error;
	return $handle;
}

########################################################################
# Start of tests
########################################################################

# To receive LDAP result messages
my $msg;

########################################################################
# Read config file
#
$conf = MyConfig::readConfig($varsFile);
ok (defined($conf->{serveruri}), "Read the vars file");

########################################################################
# Construct DNs
#
my $ApeopleDN = "dc=people,dc=a,dc=example,dc=org";
my $BpeopleDN = "dc=people,dc=b,dc=example,dc=org";
#
# And for groups
#
my $AgroupsDN = "dc=groups,dc=a,dc=example,dc=org";
my $BgroupsDN = "dc=groups,dc=b,dc=example,dc=org";
#
# Principals
#
my $UserA1 = "uniqueIdentifier=a1,$ApeopleDN";
my $UserA2 = "uniqueIdentifier=a2,$ApeopleDN";
my $UserB1 = "uniqueIdentifier=b1,$BpeopleDN";
my $UserB2 = "uniqueIdentifier=b2,$BpeopleDN";


########################################################################
# Setting up LDAP connections
########################################################################

########################################################################
# Open an LDAP connection and bind anonymously
#
my $ldapAnon = Net::LDAP->new( $conf->{serveruri},
                        version => 3,
			onerror => undef );

die "Cannot connect to LDAP server " . $conf->{serveruri} . " so tests abandoned" if not $ldapAnon;

$msg = $ldapAnon->bind;
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "Bind anonymously" );


########################################################################
# Read base object as ANON - it must contain our base OC
#
$msg = $ldapAnon->search(
        base => $conf->{basedn},
        filter => "(objectClass=organization)",
        scope => "base",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Reading the base object from the DIT");
my $baseObject = $msg->entry(0);

########################################################################
# LDAP connect
#

my $ldapMgr = newConn( "DSA Manager", $conf->{manager}, $conf->{password} );
my $ldapUserA1 = newConn( "User A1", $UserA1, "secret" );
my $ldapUserA2 = newConn( "User A2", $UserA2, "secret" );
my $ldapUserB1 = newConn( "User B1", $UserB1, "secret" );
my $ldapUserB2 = newConn( "User B2", $UserB2, "secret" );

# We may want to inspect the entry so this will be a pointer to it
my $foundObject;

########################################################################
# Anon should be able to read the department entries
#

$msg = $ldapAnon->search( base => "dc=a,dc=example,dc=org", scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Anon reading dc=a,dc=example,dc=org");
$foundObject = $msg->entry(0);

$msg = $ldapAnon->search( base => "dc=b,dc=example,dc=org", scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Anon reading dc=b,dc=example,dc=org");
$foundObject = $msg->entry(0);

########################################################################
# Entry A1 is public so all should see it
#

$msg = $ldapUserA1->search( base => $UserA1, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User A1 reading entry A1");
$foundObject = $msg->entry(0);

ok (($foundObject->get_value("uid") eq "a1"), "User A1 should be able to read own uid");
ok ((not $foundObject->get_value("userPassword")), "User A1 should not be able to read own password");

$msg = $ldapUserA2->search( base => $UserA1, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User A2 reading entry A1");
$foundObject = $msg->entry(0);

$msg = $ldapUserB1->search( base => $UserA1, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User B1 reading entry A1");
$foundObject = $msg->entry(0);

$msg = $ldapAnon->search( base => $UserA1, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Anon reading entry A1");
$foundObject = $msg->entry(0);
ok (($foundObject and $foundObject->get_value("cn")), "Anon should see cn attribute of entry A1");
# print "CN: ", $foundObject->get_value("cn"), "\n";
ok (($foundObject and not $foundObject->get_value("telephoneNumber")), "Anon should not see telephoneNumber attribute of entry A1");
ok (($foundObject and not $foundObject->get_value("uid")), "Anon should not see uid attribute of entry A1");
ok (($foundObject and not $foundObject->get_value("description")), "Anon should not see description attribute of entry A1");

$msg = $ldapAnon->search( base => $conf->{basedn}, scope => "subtree", filter => "(cn=Aye One)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "Anon searching for entry A1");
$foundObject = $msg->entry(0);
ok (($foundObject and $foundObject->get_value("cn")), "Anon should see cn attribute of entry A1");
# print "CN: ", $foundObject->get_value("cn"), "\n";

# Check which attributes in A1 are visible to authenticated users: other-dept user
$msg = $ldapUserB1->search( base => $UserA1, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User B1 reading entry A1");
$foundObject = $msg->entry(0);
ok (($foundObject and $foundObject->get_value("cn")), "User B1 should see cn attribute of entry A1");
# print "CN: ", $foundObject->get_value("cn"), "\n";
ok (($foundObject and $foundObject->get_value("telephoneNumber")), "User B1 should see telephoneNumber attribute of entry A1");
ok (($foundObject and not $foundObject->get_value("uid")), "User B1 should not see uid attribute of entry A1");
ok (($foundObject and not $foundObject->get_value("description")), "User B1 should not see description attribute of entry A1");

# Check which attributes in A1 are visible to authenticated users: same-dept user
$msg = $ldapUserA2->search( base => $UserA1, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User A2 reading entry A1");
$foundObject = $msg->entry(0);
ok (($foundObject and $foundObject->get_value("cn")), "User A2 should see cn attribute of entry A1");
# print "CN: ", $foundObject->get_value("cn"), "\n";
ok (($foundObject and $foundObject->get_value("telephoneNumber")), "User A2 should see telephoneNumber attribute of entry A1");
ok (($foundObject and $foundObject->get_value("uid")), "User A2 should see uid attribute of entry A1");
ok (($foundObject and not $foundObject->get_value("description")), "User A2 should not see description attribute of entry A1");

# Check which attributes in A1 are visible to authenticated users: self
$msg = $ldapUserA1->search( base => $UserA1, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User A1 reading entry A1");
$foundObject = $msg->entry(0);
ok (($foundObject and $foundObject->get_value("cn")), "User A1 should see cn attribute of entry A1");
# print "CN: ", $foundObject->get_value("cn"), "\n";
ok (($foundObject and $foundObject->get_value("telephoneNumber")), "User A1 should see telephoneNumber attribute of entry A1");
ok (($foundObject and $foundObject->get_value("uid")), "User A1 should see uid attribute of entry A1");
ok (($foundObject and $foundObject->get_value("description")), "User A1 should not see description attribute of entry A1");

########################################################################
# Entry A2 is visible to authenticated users
#

$msg = $ldapUserA1->search( base => $UserA2, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User A1 reading entry A2");
$foundObject = $msg->entry(0);

$msg = $ldapUserA2->search( base => $UserA2, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User A2 reading entry A2");
$foundObject = $msg->entry(0);

$msg = $ldapUserB1->search( base => $UserA2, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User B1 reading entry A2");
$foundObject = $msg->entry(0);

$msg = $ldapAnon->search( base => $UserA2, scope => "base", filter => "(objectClass=*)",);
# carp $msg->error_name if $msg->is_error;
ok ($msg->count == 0, "Anon should not be able to read entry A2");
ok (($msg->error_name eq "LDAP_NO_SUCH_OBJECT"), "Anon should not be able to detect entry A2");

########################################################################
# Entry B1 is visible to members of the same dept
#

$msg = $ldapUserA1->search( base => $UserB1, scope => "base", filter => "(objectClass=*)",);
# carp $msg->error if $msg->is_error;
ok ($msg->count == 0, "User A1 should not be able to read entry B1");
ok (($msg->error_name eq "LDAP_NO_SUCH_OBJECT"), "User A1 should not be able to detect entry B1");

$msg = $ldapUserA2->search( base => $UserB1, scope => "base", filter => "(objectClass=*)",);
# carp $msg->error if $msg->is_error;
ok ($msg->count == 0, "User A2 should not be able to read entry B1");
ok (($msg->error_name eq "LDAP_NO_SUCH_OBJECT"), "User A2 should not be able to detect entry B1");

$msg = $ldapUserB1->search( base => $UserB1, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User B1 reading entry B1");
$foundObject = $msg->entry(0);

$msg = $ldapUserB2->search( base => $UserB1, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User B2 reading entry B1");
$foundObject = $msg->entry(0);

$msg = $ldapAnon->search( base => $UserB1, scope => "base", filter => "(objectClass=*)",);
# carp $msg->error if $msg->is_error;
ok ($msg->count == 0, "Anon should not be able to read entry B1");

########################################################################
# Entry B2 has no visibility to others
#

$msg = $ldapUserA1->search( base => $UserB2, scope => "base", filter => "(objectClass=*)",);
# carp $msg->error if $msg->is_error;
ok ($msg->count == 0, "User A1 should not be able to read entry B2");
ok (($msg->error_name eq "LDAP_NO_SUCH_OBJECT"), "User A1 should not be able to detect entry B2");

$msg = $ldapUserA2->search( base => $UserB2, scope => "base", filter => "(objectClass=*)",);
# carp $msg->error if $msg->is_error;
ok ($msg->count == 0, "User A2 should not be able to read entry B2");
ok (($msg->error_name eq "LDAP_NO_SUCH_OBJECT"), "User A2 should not be able to detect entry B2");

$msg = $ldapUserB1->search( base => $UserB2, scope => "base", filter => "(objectClass=*)",);
# carp $msg->error if $msg->is_error;
ok ($msg->count == 0, "User B1 should not be able to read entry B2");
ok (($msg->error_name eq "LDAP_NO_SUCH_OBJECT"), "User B1 should not be able to detect entry B2");

$msg = $ldapUserB2->search( base => $UserB2, scope => "base", filter => "(objectClass=*)",);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User B2 reading entry B2");
$foundObject = $msg->entry(0);

$msg = $ldapAnon->search( base => $UserB2, scope => "base", filter => "(objectClass=*)",);
# carp $msg->error if $msg->is_error;
ok ($msg->count == 0, "Anon should not be able to read entry B2");
ok (($msg->error_name eq "LDAP_NO_SUCH_OBJECT"), "Anon should not be able to detect entry B2");


########################################################################
# User A1 searching for an entry in same dept
#
$msg = $ldapUserA1->search(
        base => $conf->{basedn},
        filter => "(uid=a2)",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 1, "User A1 should be able to find user a2 by uid");
$foundObject = $msg->entry(0);

ok (($foundObject->get_value("uid") eq "a2"), "User A1 should be able to read a2's uid");
ok ((not $foundObject->get_value("userPassword")), "User A1 should not be able to read a2's password");


########################################################################
# Anon searching on uid
#
$msg = $ldapAnon->search(
        base => $conf->{basedn},
        filter => "(uid=a1)",
);
carp $msg->error if $msg->is_error;
ok ($msg->count == 0, "Anon should not be able to search for an account by uid");


########################################################################
# UserA1 setting own password
#

my $entry = Net::LDAP::Entry->new;
$entry->dn($UserA1);
$entry->changetype("modify");
$entry->replace( userPassword => "mine!" );
$msg = $entry->update( $ldapUserA1 );
carp $msg->error if $msg->is_error;
ok ( !$msg->is_error, "User A1 setting own password" );

# Set it back again
$entry->replace( userPassword => "secret" );
$msg = $entry->update( $ldapMgr );
carp "ldapAdm: " . $msg->error if $msg->is_error;

